import java.awt.Graphics;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;

public class Gravity {
	static class MyFrame extends JFrame {

		private static final long serialVersionUID = 1L;
		long framesDrawed;
		int col = 0;

		int j = 100000;
		short[] h = new short[25600];
		int N = 8; // The number of centers of gravity.

		int k[] = new int[N * 2];

		int[] raster;
		ColorModel cm;
		DataBuffer buffer;
		SampleModel sm;
		WritableRaster wrRaster;
		BufferedImage backBuffer;

		public void initArr() {
			for (int w = 0; w < 25600; w++)
				h[w] = (short) (128 + 128 * Math.sin((w) / 256.0)); 
			for (int w = 0; w < N * 2; w++)
				k[w] = 20 + (int) (Math.random() * 200);
		}

		// @Override public void paint(Graphics g)
		public void draw(Graphics g) {
			// reinitialize all if resized
			int width = getWidth();
			int height = getHeight();
			this.setSize(width,height);
			int size = width * height;
			raster = new int[size];  // https://stackoverflow.com/questions/33460365/what-the-fastest-way-to-draw-pixels-buffer-in-java
			cm = new DirectColorModel(24, 255, 255 << 8, 255 << 16); 
			buffer = new DataBufferInt(raster, size);
			sm = cm.createCompatibleSampleModel(width, height);
			wrRaster = Raster.createWritableRaster(sm, buffer, null);
			backBuffer  = new BufferedImage(cm, wrRaster, false, null);
			double[] XP = new double[N];
			double[] YP = new double[N];
			double[] D  = new double[N];
			
			for (int w = 0; w < N; w++) {
				XP[w] = width  / 2 - (Math.sin((double) j / k[2 * w])     * (double) width  / 2);
				YP[w] = height / 2 - (Math.cos((double) j / k[2 * w + 1]) * (double) height / 2);
			}

			for (int pos = 0; pos < size; pos++) {

				int X = pos % width;
				int Y = pos / width;

				for (int w = 0; w < N; w++)
					D[w] = Math.sqrt((XP[w] - X) * (XP[w] - X) + (YP[w] - Y) * (YP[w] - Y));
				double S1 = 0, S2 = 0;
				for (int w = 0; w < N / 2; w++)
					S1 += D[w];
				for (int w = 0; w < N; w++)
					S2 += D[w];
				double d = S1 / S2;

				raster[pos] = ((int) (h[(int) (d * 11291)]) << 16) | ((int) (h[(int) (d * 17875)]) << 8) | ((int) (h[(int) (d * 6452)]));
			}
			g.drawImage(backBuffer, 0, 0, null);
			++framesDrawed;
			j++;

		}
	}

	public static void main(String[] args) {
		final MyFrame frame = new MyFrame();
		frame.initArr();
		frame.setSize(800, 600);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);

		// draw FPS in title
		new Timer(1000, new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				frame.setTitle("Performance = " + Long.toString(frame.framesDrawed) + " frames per second");
				frame.framesDrawed = 0;
			}
		}).start();

		frame.createBufferStrategy(1);
		BufferStrategy bs = frame.getBufferStrategy();
		Graphics g = bs.getDrawGraphics();
		for (;;) frame.draw(g);
	}
}